#![allow(dead_code)]

mod tag;
mod util;
mod runtime;
mod interface; pub use interface::*;
mod encoding; pub use encoding::*;

pub fn test() {
    use std::fs::File;
    use std::io::prelude::*;

    const MAGAZINE :usize     = 0x100;
    const MAGAZINE_ROW :usize = 0x101;

    // define schema "Magazine Row"
    Schema::with(vec![
        ("Content", natural()),
        ("Quantity", natural()),
    ]).bind(MAGAZINE_ROW);

    // define schema "Magazine"
    Schema::with(vec![
        ("Capacity", natural()),
        ("Quantity", natural()),
        ("Content", list(record(MAGAZINE_ROW))),
    ]).bind(MAGAZINE);

    // create record "Magazine"
    let data = Record::with(MAGAZINE, vec![
        ("Capacity", *Natural::with(30)),
        ("Quantity", *Natural::with(25)),
        ("Content", *List::with(record(MAGAZINE_ROW), vec![
            *Record::with(MAGAZINE_ROW, vec![
                ("Content", *Natural::with(15)),
                ("Quantity", *Natural::with(5)),
            ]).unwrap(),
            *Record::with(MAGAZINE_ROW, vec![
                ("Content", *Natural::with(16)),
                ("Quantity", *Natural::with(20)),
            ]).unwrap(),
        ])),
    ]).unwrap();

    // encode record
    let out = encode(*data);

    // write encoding to file
    let mut file = File::create("dat.szn").unwrap();
    file.write_all(&out).ok();

    // print hex series of encoding
    print!("[{}]: ", out.len());
    for byte in &out {
        print!("{:02x} ", *byte);
    } println!("");
}

#[cfg(test)]
mod tests;
